# encoding: utf-8

from pathlib import Path

import os
import httpretty

from httpretty import httprettified
from opencage.geocoder import OpenCageGeocode

# reduce maximum backoff retry time from 120s to 1s
os.environ['BACKOFF_MAX_TIME'] = '1'


geocoder = OpenCageGeocode('abcde')

@httprettified
def test_gb_postcode():
    httpretty.register_uri(
        httpretty.GET,
        geocoder.url,
        body=Path('test/fixtures/uk_postcode.json').read_text()
    )

    results = geocoder.geocode("EC1M 5RF")
    assert any((abs(result['geometry']['lat'] - 51.5201666) < 0.05 and abs(result['geometry']['lng'] - -0.0985142) < 0.05) for result in results)


@httprettified
def test_australia():
    httpretty.register_uri(
        httpretty.GET,
        geocoder.url,
        body=Path('test/fixtures/mudgee_australia.json').read_text()
    )

    results = geocoder.geocode("Mudgee, Australia")
    assert any((abs(result['geometry']['lat'] - -32.5980702) < 0.05 and abs(result['geometry']['lng'] - 149.5886383) < 0.05) for result in results)


@httprettified
def testMunster():
    httpretty.register_uri(
        httpretty.GET,
        geocoder.url,
        body=Path('test/fixtures/muenster.json').read_text()
    )

    results = geocoder.geocode("Münster")
    assert any((abs(result['geometry']['lat'] - 51.9625101) < 0.05 and abs(result['geometry']['lng'] - 7.6251879) < 0.05) for result in results)

@httprettified
def testDonostia():
    httpretty.register_uri(
        httpretty.GET,
        geocoder.url,
        body=Path('test/fixtures/donostia.json').read_text()

    )

    results =geocoder.geocode("Donostia")
    assert any((abs(result['geometry']['lat'] - 43.300836) < 0.05 and abs(result['geometry']['lng'] - -1.9809529) < 0.05) for result in results)

    # test that the results are in unicode
    assert results[0]['formatted'] == 'San Sebastián, Autonomous Community of the Basque Country, Spain'



# encoding: utf-8

from pathlib import Path

import httpretty
import pytest

from httpretty import httprettified
from opencage.geocoder import OpenCageGeocode
from opencage.geocoder import NotAuthorizedError


@httprettified
def test_success():
    with OpenCageGeocode('abcde') as geocoder:
        httpretty.register_uri(
            httpretty.GET,
            geocoder.url,
            body=Path('test/fixtures/uk_postcode.json').read_text()
        )

        results = geocoder.geocode("EC1M 5RF")
        assert any((abs(result['geometry']['lat'] - 51.5201666) < 0.05 and abs(result['geometry']['lng'] - -0.0985142) < 0.05) for result in results)

@httprettified
def test_failure():
    with OpenCageGeocode('unauthorized-key') as geocoder:
        httpretty.register_uri(
            httpretty.GET,
            geocoder.url,
            body=Path('test/fixtures/401_not_authorized.json').read_text(),
            status=401,
        )

        with pytest.raises(NotAuthorizedError) as excinfo:
            geocoder.geocode("whatever")
        assert str(excinfo.value) == 'Your API key is not authorized. You may have entered it incorrectly.'



from pathlib import Path

import datetime
import httpretty
import pytest

from httpretty import httprettified
from opencage.geocoder import OpenCageGeocode
from opencage.geocoder import RateLimitExceededError

geocoder = OpenCageGeocode('abcde')

@httprettified
def test_no_rate_limit():
    httpretty.register_uri(
        httpretty.GET,
        geocoder.url,
        body=Path('test/fixtures/no_ratelimit.json').read_text()
    )
    # shouldn't raise an exception
    geocoder.geocode("whatever")


@httprettified
def test_rate_limit_exceeded():
    # 4372eff77b8343cebfc843eb4da4ddc4 will always return 402
    httpretty.register_uri(
        httpretty.GET,
        geocoder.url,
        body=Path('test/fixtures/402_rate_limit_exceeded.json').read_text(),
        status=402,
        adding_headers={'X-RateLimit-Limit': '2500', 'X-RateLimit-Remaining': '0', 'X-RateLimit-Reset': '1402185600'},
    )

    with pytest.raises(RateLimitExceededError) as excinfo:
        geocoder.geocode("whatever")
    assert str(excinfo.value) == 'Your rate limit has expired. It will reset to 2500 on 2021-03-08T00:00:00'
    assert excinfo.value.reset_to == 2500
    assert excinfo.value.reset_time == datetime.datetime(2021, 3, 8, 0, 0)
